/*-
 * Copyright (c) 2011 aian
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#ifndef QIE__PGM_READER_H
#define QIE__PGM_READER_H

#include <string>
#include <libqie/pnm_common.h>

namespace qie {

class gray_image;
class pnm_header;

class pgm_reader {
public:
  pgm_reader();
  virtual ~pgm_reader();
private:  // no implement
  pgm_reader(const pgm_reader& other);
  pgm_reader& operator=(const pgm_reader& rhs);
public:
  void read(const std::string& fname, gray_image& image);
private:
  void parse_header(std::ifstream& ifs, pnm_header& header) const;
  void parse_format_type(std::ifstream& ifs, pnm_header& header) const;
  void parse_image_size(std::ifstream& ifs, pnm_header& header) const;
  void parse_pixel_depth(std::ifstream& ifs, pnm_header& header) const;
  void parse_ascii_pixels(std::ifstream& ifs, gray_image& image) const;
  void parse_binary_pixels(std::ifstream& ifs, gray_image& image) const;
private:
  void prepare_image(const pnm_header& header, gray_image& image) const;
private:
  void get_token(std::ifstream& ifs, std::string& token) const;
  void skip_until_newline(std::ifstream& ifs) const;
};

}

#endif  // QIE__PGM_READER_H
