/*-
 * Copyright (c) 2011 aian
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <cassert>
#include <algorithm>
#include <libqie/pnm_header.h>

namespace qie {

pnm_header::pnm_header()
  : format_(pnm_format_unknown), width_(0), height_(0), depth_(0) {
}

pnm_header::pnm_header(const pnm_header& other)
  : format_(other.format_), width_(other.width_), height_(other.height_),
    depth_(other.depth_) {
}

pnm_header::~pnm_header() {
}

pnm_header&
pnm_header::operator=(const pnm_header& rhs) {
  if (this != &rhs) {
    pnm_header temp(rhs);
    temp.swap(*this);
  }
  return *this;
}

void
pnm_header::set_width(int width) throw () {
  width_ = width;
}

void
pnm_header::set_height(int height) throw () {
  height_ = height;
}

void
pnm_header::set_depth(int depth) throw () {
  depth_ = depth;
}

void
pnm_header::set_format(const pnm_format_t& format) throw () {
  format_ = format;
}

void
pnm_header::set_format(const std::string& format) throw () {
  if (format == "P1") {
    format_ = pnm_format_ascii_bitmap;
  } else if (format == "P2") {
    format_ = pnm_format_ascii_graymap;
  } else if (format == "P3") {
    format_ = pnm_format_ascii_pixmap;
  } else if (format == "P4") {
    format_ = pnm_format_binary_bitmap;
  } else if (format == "P5") {
    format_ = pnm_format_binary_graymap;
  } else if (format == "P6") {
    format_ = pnm_format_binary_pixmap;
  } else {
    format_ = pnm_format_unknown;
  }
}

pnm_format_t
pnm_header::format() const throw () {
  return format_;
}

std::string
pnm_header::format_string() const throw () {
  std::string str;
  switch (format_) {
  case pnm_format_ascii_bitmap:
    str = "P1";
    break;
  case pnm_format_ascii_graymap:
    str = "P2";
    break;
  case pnm_format_ascii_pixmap:
    str = "P3";
    break;
  case pnm_format_binary_bitmap:
    str = "P4";
    break;
  case pnm_format_binary_graymap:
    str = "P5";
    break;
  case pnm_format_binary_pixmap:
    str = "P6";
    break;
  default:
    assert(false);
    str = "[UNKNOWN]";
  }
  return str;
}

int
pnm_header::width() const throw () {
  return width_;
}

int
pnm_header::height() const throw () {
  return height_;
}

int
pnm_header::depth() const throw () {
  return depth_;
}

void
pnm_header::swap(pnm_header& other) throw () {
  std::swap(format_, other.format_);
  std::swap(width_, other.width_);
  std::swap(height_, other.height_);
  std::swap(depth_, other.depth_);
}

}
